/*
;---FUNCTION-----------------------------------------------------------------
;
;   function    : This file makes it possible to use inp(), inpw(),
;                 outp(), outpw() on Linux Kernel 2.6.11.4 or later.
;
;---END OF FILE HEADER-------------------------------------------------------
*/

#include <sys/io.h>  //für iopl
#include <sys/mman.h>
#include <fcntl.h>
#include <ctype.h>
#include "typedefs.h"

#ifndef __STATLIBLINUXINC_C
#define __STATLIBLINUXINC_C

#define IOPLI iopl(3)
#define IOPLO iopl(0)

/***** codes ***************************************************************/

/************************************************************************/
/*                                inp                                   */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* read a byte from a port                                              */
/*                                                                      */
/* Arguments:                                                           */
/*   INT port       input port                                          */
/*                                                                      */
/* Return Value:                                                        */
/*   INT            value which was read from specified port            */
/*                                                                      */
/************************************************************************/
INT inp (UINT port)
{
	INT value;
	IOPLI;
	value = inb(port);
	IOPLO;
	return value;
}

/************************************************************************/
/*                               inpw                                   */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* read a word from a port                                              */
/*                                                                      */
/* Arguments:                                                           */
/*   UINT port      output port                                         */
/*                                                                      */
/* Return Value:                                                        */
/*   UINT           value which was read from specified port            */
/*                                                                      */
/************************************************************************/
UINT inpw (UINT port)
{
	UINT value;
	IOPLI;
	value = inw(port);
	IOPLO;
	return value;
}

/************************************************************************/
/*                              outp                                    */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* output a byte at a port                                              */
/*                                                                      */
/* Arguments:                                                           */
/*   INT  port      output port                                         */
/*   UINT value     value to write                                      */
/*                                                                      */
/* Return Value:                                                        */
/*   INT            value which was written at a port                   */
/*                                                                      */
/************************************************************************/
INT outp (UINT port, INT value)
{
	IOPLI;
	outb(value, port);
	IOPLO;
	return (INT) value;
}

/************************************************************************/
/*                              outpw                                   */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* output a word at a port                                              */
/*                                                                      */
/* Arguments:                                                           */
/*   UINT port      output port                                         */
/*   UINT value     value to write                                      */
/*                                                                      */
/* Return Value:                                                        */
/*   UINT           value which was written at a port                   */
/*                                                                      */
/************************************************************************/
UINT outpw (UINT port, UINT value)
{
	IOPLI;
	outw(value, port);
	IOPLO;
	return (UINT) value;
}

/************************************************************************/
/*                                MapMem                                */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* Function search in BIOS memory for BIOS identifikation string. If    */
/* String was found it will be copied to PCHAR IdString.                */
/*                                                                      */
/* Arguments:                                                           */
/*   PCHAR IdString Pointer to buffer                                   */
/*   UINT  size     Size of buffer                                      */
/*                                                                      */
/* Return Value:                                                        */
/*   BOOL           True if function successfully ends otherwise false. */
/*                                                                      */
/************************************************************************/
PVOID MapMem(PVOID StartAddress, ULONG Length)
{
  PVOID retValue = (PVOID) 0;
	INT fd = 0;

  //Open memory
	if ((fd = open("/dev/mem", O_RDONLY)) == -1) 
		{
		return retValue;	//Return if open memory failed
    }
    // Map memory
	if ((retValue = mmap(NULL, Length, PROT_READ, MAP_SHARED, fd,(off_t) StartAddress)) == MAP_FAILED)
		{
		return (PVOID) 0;	//Return if mapping failed
    }
	close(fd);					//Close memory
	return retValue;		//Return pointer
}

/************************************************************************/
/*                              FreeMem                                 */
/************************************************************************/
/*                                                                      */
/* Routine Description:                                                 */
/*                                                                      */
/* Function frees mapped memory.                                        */
/*                                                                      */
/* Arguments:                                                           */
/*   PVOID StartAddress    Address of physical memory                   */
/*   PVOID VirtualAddress  Address which was returned by MapMem()       */
/*   ULONG Length          Length of mapped memory                      */
/*                                                                      */
/* Return Value:                                                        */
/*   BOOL           True if function successfully ends otherwise false. */
/*                                                                      */
/************************************************************************/
BOOL FreeMem(PVOID StartAddress, PVOID VirtualAddress, ULONG Length)
	{
	return (munmap(StartAddress, Length) ? FALSE : TRUE);
	}

#endif

