/*
Linux Driver for Enterpoint's Raggedstone1 FPGA PCI Board
This demo driver allows access to the Board's 7segment displays.

by Joachim Franek 2009-11-24

This work is based on:

    Linux Driver for Enterpoint's Raggedstone1 FPGA PCI Board
    This demo driver allows access to the Board's 7segment displays.

    Written by Manuel Bessler <m.bessler AT gmx.net>
    Based on examples from the Linux Kernel Module Programming Guide
    
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or 
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/fs.h>
#include <linux/ioctl.h>
#include <asm/uaccess.h>

MODULE_AUTHOR("Joachim Franek");
MODULE_DESCRIPTION("Enterpoint Raggedstone1 FPGA PCI Development Board Driver");
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("Raggedstone1 FPGA PCI Board");


#define VENDOR_ID_ALTER  A 0x1172
#define DEVICE_ID_ALTERA 0x0100
#define VENDOR_ID_XILINX 0x10ee
#define DEVICE_ID_XILINX 0x2010


#define MAJOR_NUM 249
#define IOCTL_SET7SEG _IOR(MAJOR_NUM, 0, u16)
#define IOCTL_GET7SEG _IOW(MAJOR_NUM, 1, u16)
#define IOCTL_SETLED  _IOR(MAJOR_NUM, 2, u16)
#define DEVICE_NAME "rs1"

#define SUCCESS 0

unsigned long memstart = 0, memend = 0, memlen = 0;
void * vaddr[2] = {0,0};
static int Device_Open = 0;
unsigned long flags;

/*------------------------------------------------------------------------*/
int rs1_device_ioctl(struct inode *inode,struct file *file,unsigned int ioctl_num,unsigned long ioctl_param) 
	{
  u16 display_val;
  //printk (KERN_INFO "rs1: device_ioctl(%p,%p,num=0x%x,param=0x%lx)\n", inode, file, ioctl_num, ioctl_param);
  //printk (KERN_INFO "rs1: device_ioctl num=0x%x param=0x%lx\n",ioctl_num, ioctl_param);

  switch (ioctl_num) 
	  {
		case IOCTL_SET7SEG:
			{
	    //printk(KERN_INFO "rs1: IOCTL_SET7SEG\n");
	    display_val = (u16) ioctl_param;
	    writew(display_val, (vaddr[0]+0x20));  //16bit schreiben auf vaddr[0](=pointer auf bar0)
	    break;
			}
		case IOCTL_GET7SEG:
			{
	    //printk(KERN_INFO "rs1: IOCTL_GET7SEG\n");
	    //display_val = (u16) ioctl_param;
	    display_val=readw((vaddr[0]+0x20));  //16bit lesen auf vaddr[0](=pointer auf bar0)
			//printk(KERN_INFO "rs1: GET7SEG:0x%x\n",display_val);
			//ioctl_param=display_val; das geht nicht
			put_user(display_val,(int *)ioctl_param);
	    break;
			}
		case IOCTL_SETLED:
			{
	    //printk(KERN_INFO "rs1: IOCTL_SETLED\n");
	    display_val = (u16) ioctl_param;
	    writew(display_val, (vaddr[0]+0x00));  //16bit schreiben auf vaddr[0](=pointer auf bar0)
	    break;
			}
    }
    return SUCCESS;
	}

/*------------------------------------------------------------------------*/
// This function is called whenever a process attempts to open the device file 
static int rs1_device_open(struct inode *inode, struct file *file)
	{
  printk ("device_open(%p)\n", file);  // We don't want to talk to two processes at the same time 
  if (Device_Open)
    return -EBUSY;
  Device_Open++;
  return SUCCESS;
	}

/*------------------------------------------------------------------------*/
static int rs1_device_close(struct inode *inode, struct file *file)
	{
  printk ("device_release(%p,%p)\n", inode, file);
  Device_Open --;
  return 0;
	}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
struct file_operations Fops = 
	{
  open: 		rs1_device_open,
  release: 	rs1_device_close,
  ioctl: 		rs1_device_ioctl
	};

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static struct pci_device_id  rs1_pci_device_id[] =
	{
	//vendor,device,subvendor,subdevice,class,class_mask,driver_data
  {VENDOR_ID_ALTERA, DEVICE_ID_ALTERA, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
  {VENDOR_ID_XILINX, DEVICE_ID_XILINX, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
  {}  /* end of list */
	};
MODULE_DEVICE_TABLE(rs1_pci, rs1_pci_device_id);

/*------------------------------------------------------------------------*/
int rs1_device_probe(struct pci_dev *dev, const struct pci_device_id *id)
	{
  int ret;
  ret = pci_enable_device(dev);
  if (ret < 0)
  	{
    printk(KERN_WARNING "rs1: unable to initialize PCI device\n");
    return ret;
  	}

  ret = pci_request_regions(dev, "rs1");
  if (ret < 0)
  	{
    printk(KERN_WARNING "rs1: unable to reserve PCI resources\n");
    pci_disable_device(dev);
    return ret;
  	}

  memstart = pci_resource_start(dev, 0); /* 0 for BAR0 */
  memend 	 = pci_resource_end(  dev, 0); /* 0 for BAR0 */
  memlen = pci_resource_len(dev, 0);
  //memlen = memend-memstart+1;
  printk(KERN_WARNING "rs1: memstart=0x%lx memend=0x%lx memlen=0x%lx\n", memstart, memend, memlen-1);

	flags=pci_resource_flags(dev,0);

	if ((flags & IORESOURCE_IO)!=0)  printk("rs1: bar0 is IORESOURCE_IO\n");
	if ((flags & IORESOURCE_MEM)!=0) printk("rs1: bar0 is IORESOURCE_MEM\n");

  vaddr[0] = ioremap(memstart, memlen); //hier wird ein Pointer, unter dem man bar0 erreicht gespeichert; readb/readw/readl writeb/writew/writel
	//readw(vaddr, );
 	//writew(vaddr, );
 	//writew(0xDEAD, vaddr+i);
  printk(KERN_INFO "rs1: device_probe successful\n");
  
  ret = register_chrdev(MAJOR_NUM, DEVICE_NAME, &Fops);
  if (ret < 0) 
  	{
    printk (KERN_INFO "%s failed with %d\n","Sorry, registering the character device rs1 ",ret);
    return ret;
  	}
  else
  	{
    printk (KERN_INFO "%s The major device number is %d.\n","rs1: Registeration is a success", MAJOR_NUM);
  	}
  return ret;
	}

/*------------------------------------------------------------------------*/
void rs1_device_remove(struct pci_dev *dev)
	{
  unregister_chrdev(MAJOR_NUM, DEVICE_NAME);
  iounmap(vaddr[0]);
	//  release_mem_region(memstart, memlen)
  pci_release_regions(dev);
  pci_disable_device(dev);
  printk(KERN_INFO "rs1: device removed\n");
	}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
struct pci_driver rs1_pci_driver =
	{
  name: 			"rs1",
  id_table: 	rs1_pci_device_id,
  probe: 			rs1_device_probe,
  remove: 		rs1_device_remove
	//suspend: 
	//resume:
	};

/*------------------------------------------------------------------------*/
int init_module(void)
	{
  printk(KERN_INFO "rs1: init\n");
	return pci_register_driver(&rs1_pci_driver);
	}

/*------------------------------------------------------------------------*/
void cleanup_module(void)
	{
  printk(KERN_INFO "rs1: cleanup\n");
  pci_unregister_driver(&rs1_pci_driver);
	}


